import pytest
from prefect.logging import disable_run_logger

from airbyte_prefect import exceptions as err
from airbyte_prefect.connections import AirbyteConnection, trigger_sync


async def example_trigger_sync_flow(connection_id, airbyte_server=None, **kwargs):
    with disable_run_logger():
        return await trigger_sync.fn(
            airbyte_server=airbyte_server, connection_id=connection_id, **kwargs
        )


async def test_successful_trigger_sync(
    mock_successful_connection_sync_calls, airbyte_server, connection_id
):
    trigger_sync_result = await example_trigger_sync_flow(
        airbyte_server=airbyte_server, connection_id=connection_id
    )

    assert type(trigger_sync_result) is dict

    assert trigger_sync_result == {
        "connection_id": "e1b2078f-882a-4f50-9942-cfe34b2d825b",
        "status": "active",
        "job_status": "succeeded",
        "job_created_at": 1650644844,
        "job_updated_at": 1650644844,
    }


async def test_cancelled_trigger_manual_sync(
    mock_cancelled_connection_sync_calls, airbyte_server, connection_id
):
    with pytest.raises(err.AirbyteSyncJobFailed):
        await example_trigger_sync_flow(
            airbyte_server=airbyte_server, connection_id=connection_id
        )


async def test_connection_sync_inactive(
    mock_inactive_sync_calls, airbyte_server, connection_id
):
    with pytest.raises(err.AirbyteConnectionInactiveException):
        await example_trigger_sync_flow(
            airbyte_server=airbyte_server, connection_id=connection_id
        )


async def test_failed_trigger_sync(
    mock_failed_connection_sync_calls, airbyte_server, connection_id
):
    with pytest.raises(err.AirbyteSyncJobFailed):
        await example_trigger_sync_flow(
            airbyte_server=airbyte_server, connection_id=connection_id
        )


async def test_bad_connection_id(
    mock_bad_connection_id_calls, airbyte_server, connection_id
):
    with pytest.raises(err.ConnectionNotFoundException):
        await example_trigger_sync_flow(
            airbyte_server=airbyte_server, connection_id=connection_id
        )


async def test_failed_health_check(
    mock_failed_health_check_calls, airbyte_server, connection_id
):
    with pytest.raises(err.AirbyteServerNotHealthyException):
        await example_trigger_sync_flow(
            airbyte_server=airbyte_server, connection_id=connection_id
        )


async def test_get_job_status_not_found(
    mock_invalid_job_status_calls, airbyte_server, connection_id
):
    with pytest.raises(err.JobNotFoundException):
        await example_trigger_sync_flow(
            airbyte_server=airbyte_server, connection_id=connection_id
        )


async def test_trigger_sync_with_kwargs(
    mock_successful_connection_sync_calls, connection_id
):
    trigger_sync_result = await example_trigger_sync_flow(
        airbyte_server_host="localhost",
        airbyte_server_port=8000,
        connection_id=connection_id,
    )

    assert type(trigger_sync_result) is dict

    assert trigger_sync_result == {
        "connection_id": "e1b2078f-882a-4f50-9942-cfe34b2d825b",
        "status": "active",
        "job_status": "succeeded",
        "job_created_at": 1650644844,
        "job_updated_at": 1650644844,
    }


async def test_airbyte_connection_instantiation(airbyte_server, connection_id):
    connection = AirbyteConnection(
        airbyte_server=airbyte_server,
        connection_id=connection_id,
    )

    assert isinstance(connection, AirbyteConnection)
    assert connection.airbyte_server == airbyte_server
    assert str(connection.connection_id) == connection_id



from . import _version

from airbyte_prefect.connections import AirbyteConnection  # noqa F401
from airbyte_prefect.server import AirbyteServer  # noqa F401

__version__ = _version.get_versions()["version"]



import pytest
from prefect import flow

from airbyte_prefect.connections import AirbyteSyncResult
from airbyte_prefect.exceptions import AirbyteSyncJobFailed
from airbyte_prefect.flows import run_connection_sync

expected_airbyte_sync_result = AirbyteSyncResult(
    created_at=1650644844,
    job_status="succeeded",
    job_id=45,
    records_synced=0,
    updated_at=1650644844,
)


async def test_run_connection_sync_standalone_success(
    airbyte_server, airbyte_connection, mock_successful_connection_sync_calls
):

    result = await run_connection_sync(airbyte_connection=airbyte_connection)

    assert result == expected_airbyte_sync_result


async def test_run_connection_sync_standalone_fail(
    airbyte_server, airbyte_connection, mock_failed_connection_sync_calls, caplog
):

    with pytest.raises(AirbyteSyncJobFailed):
        await run_connection_sync(airbyte_connection=airbyte_connection)


async def test_run_connection_sync_standalone_cancel(
    airbyte_server, airbyte_connection, mock_cancelled_connection_sync_calls
):

    with pytest.raises(AirbyteSyncJobFailed):
        await run_connection_sync(airbyte_connection=airbyte_connection)


async def test_run_connection_sync_standalone_status_updates(
    airbyte_server, airbyte_connection, mock_successful_connection_sync_calls, caplog
):
    airbyte_connection.status_updates = True
    await run_connection_sync(airbyte_connection=airbyte_connection)

    assert "Job 45 succeeded" in caplog.text


async def test_run_connection_sync_subflow_synchronously(
    airbyte_server, airbyte_connection, mock_successful_connection_sync_calls
):
    @flow
    def airbyte_sync_sync_flow():
        return run_connection_sync(airbyte_connection=airbyte_connection)

    result = airbyte_sync_sync_flow()

    assert result == expected_airbyte_sync_result


async def test_run_connection_sync_subflow_asynchronously(
    airbyte_server, airbyte_connection, mock_successful_connection_sync_calls
):
    @flow
    async def airbyte_sync_sync_flow():
        return await run_connection_sync(airbyte_connection=airbyte_connection)

    result = await airbyte_sync_sync_flow()

    assert result == expected_airbyte_sync_result



"""Flows for interacting with Airbyte."""

from prefect import flow, task

from airbyte_prefect.connections import AirbyteConnection, AirbyteSyncResult


@flow
async def run_connection_sync(
    airbyte_connection: AirbyteConnection,
) -> AirbyteSyncResult:
    """A flow that triggers a sync of an Airbyte connection and waits for it to complete.

    Args:
        airbyte_connection: `AirbyteConnection` representing the Airbyte connection to
            trigger and wait for completion of.

    Returns:
        `AirbyteSyncResult`: Model containing metadata for the `AirbyteSync`.

    Example:
        Define a flow that runs an Airbyte connection sync:
        ```python
        from prefect import flow
        from airbyte_prefect.server import AirbyteServer
        from airbyte_prefect.connections import AirbyteConnection
        from airbyte_prefect.flows import run_connection_sync

        airbyte_server = AirbyteServer(
            server_host="localhost",
            server_port=8000
        )

        connection = AirbyteConnection(
            airbyte_server=airbyte_server,
            connection_id="<YOUR-AIRBYTE-CONNECTION-UUID>"
        )

        @flow
        def airbyte_sync_flow():
            # do some things

            airbyte_sync_result = run_connection_sync(
                airbyte_connection=connection
            )
            print(airbyte_sync_result.records_synced)

            # do some other things, like trigger DBT based on number of new raw records
        ```
    """

    # In Prefect 3, we can directly await sync_compatible methods in async contexts
    airbyte_sync = await airbyte_connection.trigger()

    await airbyte_sync.wait_for_completion()

    return await airbyte_sync.fetch_result()
